"""The different prompt templates used in EuroEval."""

from .classification import CLASSIFICATION_TEMPLATES
from .linguistic_acceptability import LA_TEMPLATES
from .multiple_choice import MULTIPLE_CHOICE_TEMPLATES
from .named_entity_recognition import NER_TEMPLATES
from .reading_comprehension import RC_TEMPLATES
from .sentiment_classification import SENT_TEMPLATES
from .simplification import SIMPL_TEMPLATES
from .summarization import SUMM_TEMPLATES
from .token_classification import TOKEN_CLASSIFICATION_TEMPLATES
